// make a soundfile with specified frequency/cycles under a cos^2 window
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sun Apr 20 18:14:44 GMT-0800 1997
// Last Modified: Sun Apr 20 18:14:47 GMT-0800 1997
// Filename:      .../sig/examples/periods/periods.cc
// Syntax:        C++ 
//

#include "Dependencies.h"

void exitUsage(char *commandName);

int main(int argc, char* argv[]) {
   if (argc != 2) exitUsage(argv[0]); 

   float frequency;
   cout << "Enter frequency for the sinewave: ";
   cin  >> frequency;

   float numCycles;
   cout << "How many cycles of the sine wave in output: ";
   cin  >> numCycles;

   float amplitude;
   cout << "What amplitude: ";
   cin  >> amplitude;

   int numSamples;
   numSamples = (int)(1.0/frequency*numCycles*44100 + 0.5);

   SoundHeader header;
   header.setHighMono();

   // Elements:
   FixedSine     sinewave(frequency, amplitude);
   FixedSine     window(frequency/numCycles/2.0);
   Multiply      multiply;
   SoundFileOut  outsound(argv[1], header);

   // Connections:
   multiply.connect(sinewave);
   multiply.connect(window);
   multiply.connect(window);
   outsound.connect(multiply);

   Action action;
   action.tick(outsound, numSamples);


   return 0;
}



void exitUsage(char* commandName) {
   cout << endl;
   cout 
   << "Creates a specified number of periods of a certain frequency under\n"
   << "a cos^2 window. Use to determine how many periods of a sound are\n"
   << "needed to accurately detect a frequency.\n";
   cout << endl;
   cout << "Usage: " << commandName << " outsound" << endl;
   cout << endl;
   cout << "Program input: " << endl;
   cout << "   (a) frequency of sinewave in Hertz" << endl;
   cout << "   (b) number of cycles for sound" << endl;
   cout << "   (c) amplitude of sound [0 .. 1]" << endl;
   cout << endl;

   exit(1);
}



