// make a whitenoise soundfile.
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Wed Apr 30 22:53:46 GMT-0800 1997
// Last Modified: Sun Apr 20 17:32:07 GMT-0800 1997
// Filename:      .../sig/examples/whitenoise/whitenoise.cc
// Syntax:        C++ 
//

// #include "Dependencies.h"
#include "WhiteNoise.h"
#include "SoundFileOut.h"
#include "Downsample.h"
#include "Action.h"

void exitUsage(char *commandName);

int main(int argc, char* argv[]) {
   if (argc != 2) exitUsage(argv[0]); 

   int numSamples;
   cout << "How many samples of noise (44.1 kHz sample rate): ";
   cin  >> numSamples;

   float amplitude;
   cout << "What max amplitude for the noise [0 .. 1]: ";
   cin  >> amplitude;

   int downSample;
   cout << "How many samples to hold each random value: ";
   cin  >> downSample;

   SoundHeader header;
   header.setHighMono();

   // Elements:
   WhiteNoise   noise(amplitude);
   SoundFileOut outsound(argv[1], header);
   Downsample   hold(downSample);

   // Connections:
   hold.connect(noise);
   outsound.connect(hold);

   Action action;
   action.tick(outsound, numSamples);

   return 0;
}



void exitUsage(char* commandName) {
   cout << endl;
   cout << "Generates whitenoise or slightly filtered whitenoise." << endl;
   cout << endl;
   cout << "Usage: " << commandName << " outsound" << endl;
   cout << endl;
   cout << "Program input: " << endl;
   cout << "   (a) time of noise in samples" << endl;
   cout << "   (b) amplitude of noise [0 .. 1]" << endl;
   cout << "   (c) random sample update (whitenoise == 1)" << endl;
   cout << endl;

   exit(1);
}



