//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Tue Mar 19 20:37:01 PST 1996
// Last Modified: Sun Dec 14 18:42:24 GMT-0800 1997
// Filename:      .../sig/src/control/MidiOutput/MidiOutput.h
// Syntax:        C++ 
// $Smake:        cc -O -g -c -I../../include %b.cc && rm %b.o
//

#ifndef _MIDIOUTPUT_H_INCLUDED
#define _MIDIOUTPUT_H_INCLUDED

#include "Array.h"
#include "MidiPort.h"
#include "Definitions.h"
#include "Timer.h"
#include <fstream.h>

class MidiOutput : public MidiPort {

   public:
      MidiOutput::MidiOutput(int port, int openAuto = 1);  
      MidiOutput::~MidiOutput();

      // MIDI output commands

      // all data sent out from the computer to MIDI cable must travel through 
      // this command in order to use the recording feature:
      void send(int byte, int flushQ = 0);

      // MIDI commands
      void     cont                (int channel, int controller, int value);
      void     pc                  (int channel, int timbre);
      void     play                (int channel, int keyno, int attack = 0);
      void     pw                  (int channel,int greaterData,int lesserData);
      void     pw                  (int channel, int tuningData);
      void     pw                  (int channel, double tuningData);
      void     sustain             (int channel, int data);

      // Maintenance and other MIDI commands
      void     off                 (int channel);
      void     off                 (void);
      void     offAll              (void);
      void     reset               (void);
      void     sysex               (Array<uchar>& data);
      void     sysex               (int data);
      void     sysex               (uchar* data, int size);
      void     recordStart         (char *filename, int format = 0);
      void     recordStop          (void);


   protected:
      char         outputRecordQ;    // boolean for recording MIDI output
      char         outputRecordType; // 0 = ascii; 1 = binary; 3 = midifile
      fstream      outputRecordFile; // location to store recording
      Array<uchar> outputBuffer;     // local output buffer for midi data
      long         lastFlushTime;    // for recording delta times between output
      static Timer timer;            // for time info 

      // private functions
      void writeFourBytes(fstream& outputFile, ulong number);
      void writeOutputAscii(void);
      void writeOutputBinary(void);
      void writeOutputMidifile(void);

};




#endif  _MIDIOUTPUT_H_INCLUDED




