//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Sun Apr 20 19:03:37 GMT-0800 1997
// Last Modified: Sun Apr 20 19:03:39 GMT-0800 1997
// Filename:      .../sig/src/Filter/Comb/Comb.cc
// Syntax:        C++ 
// $Smake:        cc -Wall -g -c %b.cc -I../../../include && rm -f %b.o
//

#include <stdlib.h>
#include "Comb.h"


//////////////////////////////
//
// Comb::Comb
//

Comb::Comb(void) {
   setName("Comb");
   outputValue = 0;
   brandname = FILTER;
   setMinFund(12);             // minimum fundamental of comb is 12 Hz.
   setScale(0.5);              // related to reverb time of sound
   setFund(440);               // fundamental frequency of comb filter
}



//////////////////////////////
//
// Comb::~Comb
//

Comb::~Comb() { }



//////////////////////////////
//
// Comb::action
//

void Comb::action(void) {
   delayline.write(getSignal() + outputValue * getScale());
   outputValue = delayline[getDelay()+1];
}



//////////////////////////////
//
// Comb::getDelay
//

sampleType Comb::getDelay(void) {
   return -getSrate()/inputs[1];
}



//////////////////////////////
//
// Comb::getMinFund
//

sampleType Comb::getMinFund(void) {
   return delayline.getSize()/getSrate();
}



//////////////////////////////
//
// Comb::getScale
//

sampleType Comb::getScale(void) {
   return inputs[2];
}



//////////////////////////////
//
// Comb::getSignal
//

sampleType Comb::getSignal(void) {
   return inputs[0];
}



//////////////////////////////
//
// Comb::output
//

sampleType Comb::output(int channel) {
   return outputValue;
}



//////////////////////////////
//
// Comb::setDelay -- sets the delay in samples
//

void Comb::setDelay(sampleType aDelay) {
   connect(-getSrate()/aDelay, 1);
}



//////////////////////////////
//
// Comb::setFund
//

void Comb::setFund(sampleType aFund) {
   connect(aFund, 1);
}



//////////////////////////////
//
// Comb::setLoopTime
//

void Comb::setLoopTime(sampleType aLoopTime) {
   connect(1.0/aLoopTime, 1);
}
   


//////////////////////////////
//
// Comb::setMinFund
//

void Comb::setMinFund(sampleType aMin) {
   delayline.setSize((long)(getSrate()/aMin));
}



//////////////////////////////
//
// Comb::setReverbTime
//

void Comb::setReverbTime(sampleType aReverbTime) {
   connect(1.0/aReverbTime, 1);
}



//////////////////////////////
//
// Comb::setScale
//

void Comb::setScale(sampleType aScale) {
   connect(aScale, 2);
}



