//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Mon Oct 13 11:34:57 GMT-0800 1997
// Last Modified: Mon Oct 13 12:51:13 GMT-0800 1997
// Filename:      .../sig/src/midi/src/Timer/Timer.cc
// Syntax:        C++ 
// $Smake:        cc -g -c %b.cc && rm -f %b.o
//


#include "Timer.h"
#include <iostream.h>


unsigned long long int Timer::globalOffset = 0;



//////////////////////////////
//
// Timer::Timer
//

Timer::Timer(void) {
   if (globalOffset == 0) {           // coordinate offset between timers
      globalOffset = clockCycles();
   }
   offset = globalOffset;
   cpuSpeed = 200000000;     // 200 MHz
   ticksPerSecond = 1000;
   calculateFactor();
}


Timer::Timer(long aSpeed) {
   if (globalOffset == 0) {
      globalOffset = clockCycles();
   }
   offset = globalOffset;
   cpuSpeed = aSpeed;
   ticksPerSecond = 1000;
   calculateFactor();
}


Timer::Timer(Timer& aTimer) {
   offset = aTimer.offset;
   cpuSpeed = aTimer.cpuSpeed;
   ticksPerSecond = aTimer.ticksPerSecond;
   conversionFactor = aTimer.conversionFactor;
}



//////////////////////////////
//
// Timer::~Timer
//

Timer::~Timer() {
   // do nothing
}



//////////////////////////////
//
// Timer::clockCycles -- returns the number of clock cycles since last reboot
//	HARDWARE DEPENDENT -- currently for Pentiums only.
//


unsigned long long int Timer::clockCycles() const {
   unsigned long long int output;
   
   // For Pentiums, you can get the number of clock cycles elapsed 
   // since the last reboot with the following assembler code:
   __asm__ volatile (".byte 0x0f, 0x31" : "=A" (output));


   return output;
}



//////////////////////////////
//
// Timer::getCpuSpeed -- returns the CPU speed of the computer.
//

long Timer::getCpuSpeed(void) const {
   return cpuSpeed;
}



//////////////////////////////
//
// Timer::getTicksPerSecond
//

long Timer::getTicksPerSecond(void) const {
   return ticksPerSecond;
}



//////////////////////////////
//
// Timer::getTime -- returns time in milliseconds by default.  
// 	time can be altered to be in some other unit of time
//	by using the setTicksPerSecond function.
//	(Default is 1000 ticks per second.)
//

millitime Timer::getTime(void) const {
   return ((clockCycles()-offset)/conversionFactor);
}
   


//////////////////////////////
//
// Timer::reset -- set the timer to 0.
//

void Timer::reset(void) {
   offset = clockCycles();
}



//////////////////////////////
//
// Timer::setCpuSpeed -- You can also vary the CPU speed here to cause the 
//	getTime function to output different units of time, but the 
//	setTicksPerSecond is a more appropriate place to do such a thing. 
//

void Timer::setCpuSpeed(long aSpeed) {
   if (aSpeed <= 0) {
      cerr << "Error: Cannot set the cpu speed to be negative: " 
           << aSpeed << endl;
      exit(1);
   }
   cpuSpeed = aSpeed;
   calculateFactor();
}



//////////////////////////////
//
// Timer::setTicksPerSecond 
//

void Timer::setTicksPerSecond(long aTickRate) {
   if (aTickRate <= 0) {
      cerr << "Error: Cannot set the tick rate to be negative: " 
           << aTickRate << endl;
      exit(1);
   }
   ticksPerSecond = aTickRate;
   calculateFactor();
}



//////////////////////////////
//
// Timer::sync 
//

void Timer::sync(Timer& aTimer) {
   offset = aTimer.offset;
}




///////////////////////////////////////////////////////////////////////////


//////////////////////////////
//
// Timer::calculateFactor 
//

void Timer::calculateFactor(void) {
   conversionFactor = (unsigned long long int)
      ((double)getCpuSpeed()/getTicksPerSecond() + 0.5);
}





